@@ -191,6 +191,9 @@
 #' * \code{fail_boots}: The indicators for failed bootstrap samples
 #'   if \code{boot} is \code{TRUE}.
 #'
+#' * \code{fail_boots_data}: The data for failed bootstrap samples
+#'   if \code{boot} is \code{TRUE}.
+#'
 #' * \code{hr_boots}: The bootstrap hazard ratio estimates if \code{boot} is
 #'   \code{TRUE}.
 #'
@@ -205,22 +208,26 @@
 #' @examples
 #'
 #' # Example 1: pooled logistic regression switching model
+#' library(dplyr)
 #' 
-#' sim1 <- tsegestsim(
-#'   n = 500, allocation1 = 2, allocation2 = 1, pbprog = 0.5, 
-#'   trtlghr = -0.5, bprogsl = 0.3, shape1 = 1.8, 
-#'   scale1 = 360, shape2 = 1.7, scale2 = 688, 
-#'   pmix = 0.5, admin = 5000, pcatnotrtbprog = 0.5, 
-#'   pcattrtbprog = 0.25, pcatnotrt = 0.2, pcattrt = 0.1, 
-#'   catmult = 0.5, tdxo = 1, ppoor = 0.1, pgood = 0.04, 
-#'   ppoormet = 0.4, pgoodmet = 0.2, xomult = 1.4188308, 
-#'   milestone = 546, outputRawDataset = 1, seed = 2000)
-#' 
+#' sim1 <- tssim(
+#'   tdxo = 1, coxo = 1, allocation1 = 1, allocation2 = 1,
+#'   p_X_1 = 0.3, p_X_0 = 0.3, 
+#'   rate_T = 0.002, beta1 = -0.5, beta2 = 0.3, 
+#'   gamma0 = 0.3, gamma1 = -0.9, gamma2 = 0.7, gamma3 = 1.1, gamma4 = -0.8,
+#'   zeta0 = -3.5, zeta1 = 0.5, zeta2 = 0.2, zeta3 = -0.4, 
+#'   alpha0 = 0.5, alpha1 = 0.5, alpha2 = 0.4, 
+#'   theta1_1 = -0.4, theta1_0 = -0.4, theta2 = 0.2,
+#'   rate_C = 0.0000855, accrualIntensity = 20/30,
+#'   followupTime = 600, fixedFollowup = 0, days = 30,
+#'   n = 500, NSim = 100, seed = 314159)
+#'   
 #' fit1 <- ipcw(
-#'   sim1$paneldata, id = "id", tstart = "tstart", 
-#'   tstop = "tstop", event = "event", treat = "trtrand", 
-#'   swtrt = "xo", swtrt_time = "xotime", base_cov = "bprog", 
-#'   numerator = "bprog", denominator = "bprog*catlag", 
+#'   sim1[[1]], id = "id", tstart = "tstart", 
+#'   tstop = "tstop", event = "Y", treat = "trtrand", 
+#'   swtrt = "xo", swtrt_time = "xotime", 
+#'   base_cov = "bprog", numerator = "bprog", 
+#'   denominator = c("bprog", "L"),
 #'   logistic_switching_model = TRUE, ns_df = 3,
 #'   swtrt_control_only = TRUE, boot = FALSE)
 #'   
@@ -271,7 +278,8 @@ ipcw <- function(data, id = "id", stratum = "", tstart = "tstart",
     p = 0
   } else {
     fml1 = formula(paste("~", paste(base_cov, collapse = "+")))
-    p = length(rownames(attr(terms(fml1), "factors")))
+    vnames = rownames(attr(terms(fml1), "factors"))
+    p = length(vnames)
   }
   
   if (p >= 1) {
@@ -294,7 +302,8 @@ ipcw <- function(data, id = "id", stratum = "", tstart = "tstart",
     p2 = 0
   } else {
     fml2 = formula(paste("~", paste(numerator, collapse = "+")))
-    p2 = length(rownames(attr(terms(fml2), "factors")))
+    vnames2 = rownames(attr(terms(fml2), "factors"))
+    p2 = length(vnames2)
   }
   
   if (p2 >= 1) {
@@ -317,7 +326,8 @@ ipcw <- function(data, id = "id", stratum = "", tstart = "tstart",
     p3 = 0
   } else {
     fml3 = formula(paste("~", paste(denominator, collapse = "+")))
-    p3 = length(rownames(attr(terms(fml3), "factors")))
+    vnames3 = rownames(attr(terms(fml3), "factors"))
+    p3 = length(vnames3)
   }
   
   if (p3 >= 1) {
@@ -370,18 +380,14 @@ ipcw <- function(data, id = "id", stratum = "", tstart = "tstart",
   df_sorted_last <- df_sorted[last_observations_indices, ]
   
   if (p >= 1) {
-    t1 = terms(formula(paste("~", paste(base_cov, collapse = "+"))))
-    t2 = attr(t1, "factors")
-    t3 = rownames(t2)
-    
-    add_vars <- setdiff(t3, varnames)
+    add_vars <- setdiff(vnames, varnames)
     if (length(add_vars) > 0) {
       out$data_outcome <- merge(out$data_outcome, 
                                 df_sorted_last[, c(id, add_vars)], 
                                 by = id, all.x = TRUE, sort = FALSE)
     }
     
-    del_vars <- setdiff(varnames, t3)
+    del_vars <- setdiff(varnames, vnames)
     if (length(del_vars) > 0) {
       out$data_outcome[, del_vars] <- NULL
     }
@@ -402,12 +408,8 @@ ipcw <- function(data, id = "id", stratum = "", tstart = "tstart",
     data1[condition, event] <- 0
     data1[condition, tstop] <- data1[condition, swtrt_time]
     
-    t1 = terms(formula(paste("~", paste(denominator, collapse = "+"))))
-    t2 = attr(t1, "factors")
-    t3 = rownames(t2)
-    
     tem_vars <- c(swtrt, swtrt_time)
-    add_vars <- c(setdiff(t3, varnames3), tem_vars)
+    add_vars <- c(setdiff(vnames3, varnames3), tem_vars)
     if (length(add_vars) > 0) {
       if (logistic_switching_model) {
         for (h in 1:K) {
@@ -433,7 +435,7 @@ ipcw <- function(data, id = "id", stratum = "", tstart = "tstart",
       }
     }
     
-    del_vars <- setdiff(varnames3, t3)
+    del_vars <- setdiff(varnames3, vnames3)
     if (length(del_vars) > 0) {
       for (h in 1:K) {
         out$data_switch[[h]]$data[, del_vars] <- NULL
