@@ -43,6 +43,11 @@
 #'   "exponential", "weibull", "lognormal", and "loglogistic" to be
 #'   modeled on the log-scale, and "normal" and "logistic" to be modeled
 #'   on the original scale.
+#' @param init A vector of initial values for the model parameters, 
+#'   including regression coefficients and the log scale parameter. 
+#'   By default, initial values are derived from an intercept-only model. 
+#'   If this approach fails, ordinary least squares (OLS) estimates, 
+#'   ignoring censoring, are used instead.
 #' @param robust Whether a robust sandwich variance estimate should be
 #'   computed. In the presence of the id variable, the score residuals
 #'   will be aggregated for each id when computing the robust sandwich
@@ -100,6 +105,8 @@
 #'
 #'     - \code{robust}: Whether the robust sandwich variance estimate
 #'       is requested.
+#'       
+#'     - \code{fail}: Whether the model fails to converge.
 #'
 #'     - \code{rep}: The replication.
 #'
@@ -209,16 +216,16 @@
 liferegr <- function(data, rep = "", stratum = "",
                      time = "time", time2 = "", event = "event",
                      covariates = "", weight = "", offset = "",
-                     id = "", dist = "weibull", robust = FALSE,
-                     plci = FALSE, alpha = 0.05, 
+                     id = "", dist = "weibull", init = NA_real_, 
+                     robust = FALSE, plci = FALSE, alpha = 0.05, 
                      maxiter = 50, eps = 1.0e-9) {
   rownames(data) = NULL
   
   elements = c(rep, stratum, covariates, weight, offset, id)
   elements = unique(elements[elements != "" & elements != "none"])
   if (!(length(elements) == 0)) {
-    mf = model.frame(formula(paste("~", paste(elements, collapse = "+"))),
-                     data = data)
+    fml = formula(paste("~", paste(elements, collapse = "+")))
+    mf = model.frame(fml, data = data, na.action = na.omit)
   } else {
     mf = model.frame(formula("~1"), data = data)
   }
@@ -229,19 +236,18 @@ liferegr <- function(data, rep = "", stratum = "",
   nvar = length(covariates)
   if (missing(covariates) || is.null(covariates) || (nvar == 1 && (
     covariates[1] == "" || tolower(covariates[1]) == "none"))) {
-    t1 = terms(formula("~1"))
     p = 0
+    t1 = terms(formula("~1"))
   } else {
-    t1 = terms(formula(paste("~", paste(covariates, collapse = "+"))))
-    t2 = attr(t1, "factors")
-    t3 = rownames(t2)
-    p = length(t3)
+    fml1 = formula(paste("~", paste(covariates, collapse = "+")))
+    p = length(rownames(attr(terms(fml1), "factors")))
+    t1 = terms(fml1)
   }
   
   if (p >= 1) {
-    mf = model.frame(t1, df)
-    xlevels = mf$xlev
-    mm = model.matrix(t1, mf)
+    mf1 <- model.frame(fml1, data = df, na.action = na.pass)
+    mm <- model.matrix(fml1, mf1)
+    xlevels = mf1$xlev
     param = colnames(mm)
     colnames(mm) = make.names(colnames(mm))
     varnames = colnames(mm)[-1]
@@ -259,8 +265,8 @@ liferegr <- function(data, rep = "", stratum = "",
   fit <- liferegcpp(data = df, rep = rep, stratum = stratum, time = time,
                     time2 = time2, event = event, covariates = varnames,
                     weight = weight, offset = offset, id = id, dist = dist,
-                    robust = robust, plci = plci, alpha = alpha, 
-                    maxiter = maxiter, eps = eps)
+                    init = init, robust = robust, plci = plci, 
+                    alpha = alpha, maxiter = maxiter, eps = eps)
   
   fit$p <- fit$sumstat$p[1]
   fit$nvar <- fit$sumstat$nvar[1]
