@@ -141,6 +141,8 @@
 #'
 #' * \code{fail}: Whether a model fails to converge.
 #'
+#' * \code{psimissing}: Whether the psi parameter cannot be estimated.
+#'
 #' * \code{settings}: A list with the following components:
 #' 
 #'     - \code{psi_test}: The survival function to calculate the Z-statistic.
@@ -191,6 +193,9 @@
 #' * \code{fail_boots}: The indicators for failed bootstrap samples
 #'   if \code{boot} is \code{TRUE}.
 #'
+#' * \code{fail_boots_data}: The data for failed bootstrap samples
+#'   if \code{boot} is \code{TRUE}.
+#'
 #' * \code{hr_boots}: The bootstrap hazard ratio estimates if \code{boot} is
 #'   \code{TRUE}.
 #'
@@ -276,7 +281,8 @@ rpsftm <- function(data, id = "id", stratum = "",
     p = 0
   } else {
     fml1 = formula(paste("~", paste(base_cov, collapse = "+")))
-    p = length(rownames(attr(terms(fml1), "factors")))
+    vnames = rownames(attr(terms(fml1), "factors"))
+    p = length(vnames)
   }
 
   if (p >= 1) {
@@ -306,28 +312,26 @@ rpsftm <- function(data, id = "id", stratum = "",
     alpha = alpha, ties = ties, tol = tol, 
     boot = boot, n_boot = n_boot, seed = seed)
   
-  out$Sstar$uid <- NULL
-  out$Sstar$ustratum <- NULL
-  out$data_outcome$uid <- NULL
-  out$data_outcome$ustratum <- NULL
-  
-  if (p >= 1) {
-    t1 = terms(formula(paste("~", paste(base_cov, collapse = "+"))))
-    t2 = attr(t1, "factors")
-    t3 = rownames(t2)
+  if (!out$psimissing) {
+    out$Sstar$uid <- NULL
+    out$Sstar$ustratum <- NULL
+    out$data_outcome$uid <- NULL
+    out$data_outcome$ustratum <- NULL
     
-    add_vars <- setdiff(t3, varnames)
-    if (length(add_vars) > 0) {
-      out$Sstar <- merge(out$Sstar, df[, c(id, add_vars)], 
-                                by = id, all.x = TRUE, sort = FALSE)
-      out$data_outcome <- merge(out$data_outcome, df[, c(id, add_vars)], 
-                                by = id, all.x = TRUE, sort = FALSE)
-    }
-    
-    del_vars <- setdiff(varnames, t3)
-    if (length(del_vars) > 0) {
-      out$Sstar[, del_vars] <- NULL
-      out$data_outcome[, del_vars] <- NULL
+    if (p >= 1) {
+      add_vars <- setdiff(vnames, varnames)
+      if (length(add_vars) > 0) {
+        out$Sstar <- merge(out$Sstar, df[, c(id, add_vars)], 
+                           by = id, all.x = TRUE, sort = FALSE)
+        out$data_outcome <- merge(out$data_outcome, df[, c(id, add_vars)], 
+                                  by = id, all.x = TRUE, sort = FALSE)
+      }
+      
+      del_vars <- setdiff(varnames, vnames)
+      if (length(del_vars) > 0) {
+        out$Sstar[, del_vars] <- NULL
+        out$data_outcome[, del_vars] <- NULL
+      }
     }
   }
   
