@@ -10,28 +10,27 @@ library(dplyr, warn.conflicts = FALSE)
 library(ggplot2)
 
 ## ----data example 1-----------------------------------------------------------
-sim1 <- tsegestsim(
-  n = 500, allocation1 = 2, allocation2 = 1, pbprog = 0.5, 
-  trtlghr = -0.5, bprogsl = 0.3, shape1 = 1.8, 
-  scale1 = 0.000025, shape2 = 1.7, scale2 = 0.000015, 
-  pmix = 0.5, admin = 5000, pcatnotrtbprog = 0.5, 
-  pcattrtbprog = 0.25, pcatnotrt = 0.2, pcattrt = 0.1, 
-  catmult = 0.5, tdxo = 1, ppoor = 0.1, pgood = 0.04, 
-  ppoormet = 0.4, pgoodmet = 0.2, xomult = 1.4188308, 
-  milestone = 546, swtrt_control_only = TRUE,
-  outputRawDataset = 1, seed = 2000)
+sim1 <- tssim(
+  tdxo = 1, coxo = 1, allocation1 = 1, allocation2 = 1,
+  p_X_1 = 0.3, p_X_0 = 0.3, 
+  rate_T = 0.002, beta1 = -0.5, beta2 = 0.3, 
+  gamma0 = 0.3, gamma1 = -0.9, gamma2 = 0.7, gamma3 = 1.1, gamma4 = -0.8,
+  zeta0 = -3.5, zeta1 = 0.5, zeta2 = 0.2, zeta3 = -0.4, 
+  alpha0 = 0.5, alpha1 = 0.5, alpha2 = 0.4, 
+  theta1_1 = -0.4, theta1_0 = -0.4, theta2 = 0.2,
+  rate_C = 0.0000855, accrualIntensity = 20/30, 
+  followupTime = 600, fixedFollowup = 0, days = 30,
+  n = 500, NSim = 100, seed = 314159)
 
 ## ----analysis example 1-------------------------------------------------------
 fit1 <- ipcw(
-  sim1$paneldata, id = "id", tstart = "tstart", 
-  tstop = "tstop", event = "died", treat = "trtrand", 
+  sim1[[1]], id = "id", tstart = "tstart", 
+  tstop = "tstop", event = "Y", treat = "trtrand", 
   swtrt = "xo", swtrt_time = "xotime", 
-  swtrt_time_lower = "timePFSobs",
-  swtrt_time_upper = "xotime_upper", base_cov = "bprog", 
-  numerator = "bprog", denominator = "bprog*catlag", 
+  base_cov = "bprog", numerator = "bprog", 
+  denominator = c("bprog", "L"),
   logistic_switching_model = TRUE, ns_df = 3,
-  relative_time = TRUE, swtrt_control_only = TRUE, 
-  boot = FALSE)
+  swtrt_control_only = TRUE, boot = FALSE)
 
 ## ----switching models example 1-----------------------------------------------
 # denominator switching model fit
