@@ -51,9 +51,11 @@ List msmcpp(
     if (p == 0 || is_true(any(is_na(match(numerator, base_cov))))) {
       stop("numerator must be a subset of base_cov");
     }
-    
-    if (is_true(any(is_na(match(numerator, denominator))))) {
-      stop("numerator must be a subset of denominator");
+  }
+  
+  if (p > 0) {
+    if (p2 == 0 || is_true(any(is_na(match(base_cov, denominator))))) {
+      stop("base_cov must be a subset of denominator");
     }
   }
   
@@ -97,11 +99,13 @@ List msmcpp(
   DataFrame u_stratum;
   IntegerVector d(p_stratum);
   IntegerMatrix stratan(n,p_stratum);
+  List levels(p_stratum);
   if (p_stratum == 1 && (stratum[0] == "" || stratum[0] == "none")) {
     has_stratum = 0;
     stratumn.fill(1);
     d[0] = 1;
     stratan(_,0) = stratumn;
+    levels[0] = 1;
   } else {
     List out = bygroup(data, stratum);
     has_stratum = 1;
@@ -109,6 +113,7 @@ List msmcpp(
     u_stratum = DataFrame(out["lookup"]);
     d = out["nlevels"];
     stratan = as<IntegerMatrix>(out["indices"]);
+    levels = out["lookups"];
   }
   
   IntegerVector stratumn_unique = unique(stratumn);
@@ -146,14 +151,24 @@ List msmcpp(
     stop("data must contain the event variable"); 
   }
   
-  if (TYPEOF(data[event]) != INTSXP && TYPEOF(data[event]) != LGLSXP) {
-    stop("event must take integer or logical values");
-  }
-  
-  IntegerVector eventnz = data[event];
-  IntegerVector eventn = clone(eventnz);
-  if (is_true(any((eventn != 1) & (eventn != 0)))) {
-    stop("event must be 1 or 0 for each subject");
+  IntegerVector eventn(n);
+  if (TYPEOF(data[event]) == LGLSXP || TYPEOF(data[event]) == INTSXP) {
+    IntegerVector eventnz = data[event];
+    if (is_true(any((eventnz != 1) & (eventnz != 0)))) {
+      stop("event must be 1 or 0 for each subject");
+    } else {
+      eventn = clone(eventnz);
+    }
+  } else if (TYPEOF(data[event]) == REALSXP) {
+    NumericVector eventnz = data[event];
+    if (is_true(any((eventnz != 1) & (eventnz != 0)))) {
+      stop("event must be 1 or 0 for each subject");
+    } else {
+      NumericVector eventnz2 = clone(eventnz);
+      eventn = as<IntegerVector>(eventnz2);
+    }
+  } else {
+    stop("event must take logical, integer, or real values");
   }
   
   if (is_true(all(eventn == 0))) {
@@ -215,14 +230,24 @@ List msmcpp(
     stop("data must contain the swtrt variable"); 
   }
   
-  if (TYPEOF(data[swtrt]) != INTSXP && TYPEOF(data[swtrt]) != LGLSXP) {
-    stop("swtrt must take integer or logical values");
-  }
-  
-  IntegerVector swtrtnz = data[swtrt];
-  IntegerVector swtrtn = clone(swtrtnz);
-  if (is_true(any((swtrtn != 1) & (swtrtn != 0)))) {
-    stop("swtrt must be 1 or 0 for each subject");
+  IntegerVector swtrtn(n);
+  if (TYPEOF(data[swtrt]) == LGLSXP || TYPEOF(data[swtrt]) == INTSXP) {
+    IntegerVector swtrtnz = data[swtrt];
+    if (is_true(any((swtrtnz != 1) & (swtrtnz != 0)))) {
+      stop("swtrt must be 1 or 0 for each subject");
+    } else {
+      swtrtn = clone(swtrtnz);
+    }
+  } else if (TYPEOF(data[swtrt]) == REALSXP) {
+    NumericVector swtrtnz = data[swtrt];
+    if (is_true(any((swtrtnz != 1) & (swtrtnz != 0)))) {
+      stop("swtrt must be 1 or 0 for each subject");
+    } else {
+      NumericVector swtrtnz2 = clone(swtrtnz);
+      swtrtn = as<IntegerVector>(swtrtnz2);
+    }
+  } else {
+    stop("swtrt must take logical, integer, or real values");
   }
   
   if (!has_swtrt_time) {
@@ -242,7 +267,6 @@ List msmcpp(
     }
   }
   
-  
   StringVector covariates(p+2);
   NumericMatrix zn(n,p);
   covariates[0] = "treated";
@@ -254,8 +278,8 @@ List msmcpp(
     if (zj == treat) {
       stop("treat should be excluded from base_cov");
     }
-    NumericVector u = data[zj];
     covariates[j+1] = zj;
+    NumericVector u = data[zj];
     zn(_,j) = u;
   }
   covariates[p+1] = "crossed";
@@ -278,18 +302,55 @@ List msmcpp(
   NumericMatrix zn_lgs_den(n,q+p2);
   if (strata_main_effect_only) {
     k = 0;
-    for (i=0; i<p_stratum; i++) {
-      for (j=0; j<d[i]-1; j++) {
-        covariates_lgs_den[k+j] = "stratum_" + std::to_string(i+1) +
-          "_level_" + std::to_string(j+1);
-        zn_lgs_den(_,k+j) = 1.0*(stratan(_,i) == j+1);
+    for (i=0; i<p_stratum; ++i) {
+      int di = d[i]-1;
+      for (j=0; j<di; ++j) {
+        covariates_lgs_den[k+j] = as<std::string>(stratum[i]);
+        if (TYPEOF(levels[i]) == STRSXP) {
+          StringVector u = levels[i];
+          std::string label = sanitize(as<std::string>(u[j]));
+          covariates_lgs_den[k+j] += label;
+        } else if (TYPEOF(levels[i]) == REALSXP) {
+          NumericVector u = levels[i];
+          covariates_lgs_den[k+j] += std::to_string(u[j]);
+        } else if (TYPEOF(levels[i]) == INTSXP 
+                     || TYPEOF(levels[i]) == LGLSXP) {
+          IntegerVector u = levels[i];
+          covariates_lgs_den[k+j] += std::to_string(u[j]);
+        }
+        zn_lgs_den(_,k+j) = (stratan(_,i) == j+1);
       }
-      k += d[i]-1;
+      k += di;
     }
   } else {
-    for (j=0; j<nstrata-1; j++) {
-      covariates_lgs_den[j] = "stratum_" + std::to_string(j+1);
-      zn_lgs_den(_,j) = 1.0*(stratumn == j+1);
+    for (j=0; j<nstrata-1; ++j) {
+      // locate the first observation in the stratum
+      int first_k = 0;
+      for (; first_k<n; ++first_k) {
+        if (stratumn[first_k] == j+1) break;
+      }
+      covariates_lgs_den[j] = "";
+      for (i=0; i<p_stratum; ++i) {
+        IntegerVector q_col = stratan(_,i);
+        int l = q_col[first_k] - 1;
+        covariates_lgs_den[j] += as<std::string>(stratum[i]);
+        if (TYPEOF(levels[i]) == STRSXP) {
+          StringVector u = levels[i];
+          std::string label = sanitize(as<std::string>(u[l]));
+          covariates_lgs_den[j] += label;
+        } else if (TYPEOF(levels[i]) == REALSXP) {
+          NumericVector u = levels[i];
+          covariates_lgs_den[j] += std::to_string(u[l]);
+        } else if (TYPEOF(levels[i]) == INTSXP 
+                     || TYPEOF(levels[i]) == LGLSXP) {
+          IntegerVector u = levels[i];
+          covariates_lgs_den[j] += std::to_string(u[l]);
+        }
+        if (i < p_stratum-1) {
+          covariates_lgs_den[j] += ".";
+        }
+      }
+      zn_lgs_den(_,j) = (stratumn == j+1);
     }
   }
   
@@ -333,14 +394,70 @@ List msmcpp(
   }
   
   
-  // order data by treat, id, and time
+  // split at treatment switching into two observations if treatment 
+  // switching occurs strictly between tstart and tstop for a subject
+  LogicalVector tosplit(n);
+  for (i=0; i<n; i++) {
+    tosplit[i] = swtrtn[i] == 1 && swtrt_timen[i] > tstartn[i] && 
+      swtrt_timen[i] < tstopn[i] ? 1 : 0;
+  }
+  
+  k = sum(tosplit);
+  if (k > 0) {
+    // copy old matrices to new matrices
+    NumericMatrix zn1(n+k, zn.ncol());
+    NumericMatrix zn_lgs_den1(n+k, zn_lgs_den.ncol());
+    for (i=0; i<n; i++) {
+      zn1(i,_) = zn(i,_);
+      zn_lgs_den1(i,_) = zn_lgs_den(i,_);
+    }
+    
+    IntegerVector sub = which(tosplit);
+    for (i=0; i<k; i++) {
+      // append a new observation by changing tstart
+      int l = sub[i];
+      idn.push_back(idn[l]);
+      stratumn.push_back(stratumn[l]);
+      tstartn.push_back(swtrt_timen[l]);
+      tstopn.push_back(tstopn[l]);
+      eventn.push_back(eventn[l]);
+      treatn.push_back(treatn[l]);
+      swtrtn.push_back(swtrtn[l]);
+      swtrt_timen.push_back(swtrt_timen[l]);
+      zn1(n+i,_) = zn(l,_);
+      zn_lgs_den1(n+i,_) = zn_lgs_den(l,_);
+      
+      // change tstop and event for the old observation
+      tstopn[l] = swtrt_timen[l];
+      eventn[l] = 0;
+    }
+    
+    // update number of rows and old matrices
+    n = n + k;
+    zn = zn1;
+    zn_lgs_den = zn_lgs_den1;
+  }
+  
+  
+  // sort data by treatment group, stratum, id, and time
   IntegerVector order = seq(0, n-1);
-  std::sort(order.begin(), order.end(), [&](int i, int j) {
-    return ((treatn[i] < treatn[j]) ||
-            ((treatn[i] == treatn[j]) && (idn[i] < idn[j])) ||
-            ((treatn[i] == treatn[j]) && (idn[i] == idn[j]) &&
-            (tstopn[i] < tstopn[j])));
-  });
+  if (has_stratum) {
+    std::sort(order.begin(), order.end(), [&](int i, int j) {
+      return ((treatn[i] < treatn[j]) ||
+              ((treatn[i] == treatn[j]) && (stratumn[i] < stratumn[j])) ||
+              ((treatn[i] == treatn[j]) && (stratumn[i] == stratumn[j]) &&
+              (idn[i] < idn[j])) ||
+              ((treatn[i] == treatn[j]) && (stratumn[i] == stratumn[j]) &&
+              (idn[i] == idn[j]) && (tstopn[i] < tstopn[j])));
+    });
+  } else {
+    std::sort(order.begin(), order.end(), [&](int i, int j) {
+      return ((treatn[i] < treatn[j]) ||
+              ((treatn[i] == treatn[j]) && (idn[i] < idn[j])) ||
+              ((treatn[i] == treatn[j]) && (idn[i] == idn[j]) && 
+              (tstopn[i] < tstopn[j])));
+    });
+  }
   
   idn = idn[order];
   stratumn = stratumn[order];
@@ -352,7 +469,7 @@ List msmcpp(
   swtrt_timen = swtrt_timen[order];
   zn = subset_matrix_by_row(zn, order);
   zn_lgs_den = subset_matrix_by_row(zn_lgs_den, order);
-  
+
   IntegerVector idx(1,0); // first observation within an id
   for (i=1; i<n; i++) {
     if (idn[i] != idn[i-1]) {
@@ -368,6 +485,20 @@ List msmcpp(
     idx1[i] = idx[i+1]-1;
   }
   
+  IntegerVector osn(n);
+  NumericVector os_timen(n);
+  for (i=0; i<nids; i++) {
+    k = idx1[i];
+    for (j=idx[i]; j<idx[i+1]; j++) {
+      osn[j] = eventn[k];
+      os_timen[j] = tstopn[k];
+    }
+  }
+  
+  if (is_true(any(ifelse(swtrtn == 1, swtrt_timen > os_timen, 0)))) {
+    stop("swtrt_time must be less than or equal to os_time");
+  }
+  
   IntegerVector stratumn1 = stratumn[idx1];
   IntegerVector treatn1 = treatn[idx1];
   NumericVector tstopn1 = tstopn[idx1];
@@ -379,8 +510,7 @@ List msmcpp(
     Named("time") = tstopn1,
     Named("event") = eventn1);
   
-  DataFrame lr = lrtest(lrdata, "", "stratum", "treat", "time", "event", 
-                        0, 0);
+  DataFrame lr = lrtest(lrdata, "", "stratum", "treat", "time", "event",0,0);
   double logRankPValue = lr["logRankPValue"];
   
   double zcrit = R::qnorm(1-alpha/2, 0, 1, 1, 0);
@@ -392,40 +522,22 @@ List msmcpp(
             covariates, covariates_lgs_num, covariates_lgs_den, 
             firth, flic, ns_df, 
             stabilized_weights, trunc, trunc_upper_only, 
-            swtrt_control_only, treat_alt_interaction, 
-            alpha, zcrit, ties](
+            swtrt_control_only, treat_alt_interaction, alpha, zcrit, ties](
                 IntegerVector& idb, IntegerVector& stratumb,
                 NumericVector& tstartb, NumericVector& tstopb,
                 IntegerVector& eventb, IntegerVector& treatb,
+                NumericVector& os_timeb, 
                 IntegerVector& swtrtb, NumericVector& swtrt_timeb,
-                NumericMatrix& zb, NumericMatrix& zb_lgs_den)->List {
+                NumericMatrix& zb, 
+                NumericMatrix& zb_lgs_den)->List {
                   int h, i, j, n = static_cast<int>(idb.size());
                   bool fail = 0; // whether any model fails to converge
                   NumericVector init(1, NA_REAL);
                   
-                  // order data by treat, id, and time
-                  IntegerVector order = seq(0, n-1);
-                  std::sort(order.begin(), order.end(), [&](int i, int j) {
-                    return ((treatb[i] < treatb[j]) ||
-                            ((treatb[i]==treatb[j]) && (idb[i] < idb[j])) ||
-                            ((treatb[i]==treatb[j]) && (idb[i]== idb[j]) &&
-                            (tstopb[i] < tstopb[j])));
-                  });
-                  
-                  idb = idb[order];
-                  stratumb = stratumb[order];
-                  tstartb = tstartb[order];
-                  tstopb = tstopb[order];
-                  eventb = eventb[order];
-                  treatb = treatb[order];
-                  swtrtb = swtrtb[order];
-                  swtrt_timeb = swtrt_timeb[order];
-                  zb = subset_matrix_by_row(zb, order);
-                  zb_lgs_den = subset_matrix_by_row(zb_lgs_den, order);
-                  
-                  // exclude observations after treatment switch
-                  IntegerVector l = which((swtrtb == 0) |
-                    (tstartb < swtrt_timeb));
+                  // exclude observations after treatment switch for logistic
+                  LogicalVector c1 = ifelse(
+                    swtrtb == 1, tstartb < swtrt_timeb, tstopb < os_timeb);
+                  IntegerVector l = which(c1);
                   IntegerVector id1 = idb[l];
                   IntegerVector stratum1 = stratumb[l];
                   NumericVector tstart1 = tstartb[l];
@@ -435,15 +547,14 @@ List msmcpp(
                   NumericVector swtrt_time1 = swtrt_timeb[l];
                   NumericMatrix z1_lgs_den = 
                     subset_matrix_by_row(zb_lgs_den, l);
-                  
-                  // set up crossover and event indicators
                   int n1 = static_cast<int>(l.size());
+                  
+                  // set up crossover indicators
                   IntegerVector cross1(n1);
                   for (i=0; i<n1; i++) {
                     if (i == n1-1 || id1[i] != id1[i+1]) {
                       if (swtrt1[i] == 1 && tstop1[i] >= swtrt_time1[i]) {
                         cross1[i] = 1;
-                        tstop1[i] = swtrt_time1[i];
                       }
                     }
                   }
@@ -499,7 +610,7 @@ List msmcpp(
                   int K = static_cast<int>(treats.size());
                   
                   // initialize weights
-                  NumericVector w1(n1, 1.0), sw1(n1, 1.0);
+                  NumericVector wb(n, 1.0), swb(n, 1.0);
                   
                   // fit the switching models by treatment group
                   for (h=0; h<K; h++) {
@@ -557,43 +668,6 @@ List msmcpp(
                     DataFrame f_den = DataFrame(fit_den["fitted"]);
                     NumericVector h_den = f_den["fitted_values"];
                     
-                    // convert to probability of remaining uncensored
-                    NumericVector pstay_den = 1.0 - h_den;
-                    
-                    // obtain cumulative products within a subject
-                    IntegerVector idx(1,0);
-                    for (i=1; i<n2; i++) {
-                      if (id2[i] != id2[i-1]) {
-                        idx.push_back(i);
-                      }
-                    }
-                    
-                    int nids2 = static_cast<int>(idx.size());
-                    idx.push_back(n2);
-                    
-                    // obtain probability of treatment for denominator
-                    NumericVector p_den(n2); 
-                    for (i=0; i<nids2; i++) {
-                      if (idx[i+1] - idx[i] == 1) {  // only one record
-                        if (cross2[idx[i]] == 1) {
-                          p_den[idx[i]] = h_den[idx[i]];
-                        } else {
-                          p_den[idx[i]] = pstay_den[idx[i]];
-                        }
-                      } else { // more than one record
-                        p_den[idx[i]] = pstay_den[idx[i]];
-                        for (j=idx[i]+1; j<idx[i+1]-1; j++) {
-                          p_den[j] = p_den[j-1]*pstay_den[j];
-                        }
-                        j = idx[i+1]-1;
-                        if (cross2[j] == 1) {
-                          p_den[j] = p_den[j-1]*h_den[j];
-                        } else {
-                          p_den[j] = p_den[j-1]*pstay_den[j];
-                        }
-                      }
-                    }
-                    
                     List fit_num = logisregcpp(
                       data1, "", "cross", covariates_lgs_num, "", "", 
                       "", "uid", "logit", init, 
@@ -606,29 +680,69 @@ List msmcpp(
                     DataFrame f_num = DataFrame(fit_num["fitted"]);
                     NumericVector h_num = f_num["fitted_values"];
                     
-                    // convert to probability of remaining uncensored
-                    NumericVector pstay_num = 1.0 - h_num;
+                    // convert to probability of observed response 
+                    NumericVector o_den(n2), o_num(n2);
+                    for (i=0; i<n2; i++) {
+                      o_den[i] = cross2[i] == 1 ? h_den[i] : 1 - h_den[i];
+                      o_num[i] = cross2[i] == 1 ? h_num[i] : 1 - h_num[i];
+                    }
                     
-                    NumericVector p_num(n2); 
-                    for (i=0; i<nids2; i++) {
-                      if (idx[i+1] - idx[i] == 1) {  // only one record
-                        if (cross2[idx[i]] == 1) {
-                          p_num[idx[i]] = h_num[idx[i]];
-                        } else {
-                          p_num[idx[i]] = pstay_num[idx[i]];
+                    // obtain cumulative products within a subject
+                    IntegerVector idx2(1,0);
+                    for (i=1; i<n2; i++) {
+                      if (id2[i] != id2[i-1]) {
+                        idx2.push_back(i);
+                      }
+                    }
+                    
+                    idx2.push_back(n2);
+                    
+                    l = which(treatb == h);
+                    IntegerVector id3 = idb[l];
+                    IntegerVector swtrt3 = swtrtb[l];
+                    int n3 = static_cast<int>(l.size());
+                    
+                    IntegerVector idx3(1,0);
+                    for (i=1; i<n3; i++) {
+                      if (id3[i] != id3[i-1]) {
+                        idx3.push_back(i);
+                      }
+                    }
+                    
+                    IntegerVector swtrt3u = swtrt3[idx3];
+                    
+                    int nids3 = static_cast<int>(idx3.size());
+                    idx3.push_back(n3);
+                    
+                    NumericVector p_den(n3, 1.0), p_num(n3, 1.0);
+                    
+                    int m = 0; // cum obs prior to current id2
+                    int v = 0; // index for current unique id2
+                    for (i=0; i<nids3; i++) {
+                      int mi = swtrt3u[i] == 1 ? idx2[v+1] - idx2[v] : 
+                      idx3[i+1] - idx3[i] - 1;
+                      int r = m - idx3[i] - 1;
+                      if (swtrt3u[i] == 1) {
+                        // cum prod before switch
+                        int jj = std::min(idx3[i]+mi, idx3[i+1]-1);
+                        for (j=idx3[i]+1; j<=jj; j++) {
+                          p_den[j] = p_den[j-1]*o_den[r+j];
+                          p_num[j] = p_num[j-1]*o_num[r+j];
                         }
-                      } else { // more than one record
-                        p_num[idx[i]] = pstay_num[idx[i]];
-                        for (j=idx[i]+1; j<idx[i+1]-1; j++) {
-                          p_num[j] = p_num[j-1]*pstay_num[j];
+                        // LOCF after switch
+                        for (j=jj+1; j<idx3[i+1]; j++) {
+                          p_den[j] = p_den[j-1];
+                          p_num[j] = p_num[j-1];
                         }
-                        j = idx[i+1]-1;
-                        if (cross2[j] == 1) {
-                          p_num[j] = p_num[j-1]*h_num[j];
-                        } else {
-                          p_num[j] = p_num[j-1]*pstay_num[j];
+                      } else {
+                        for (j=idx3[i]+1; j<idx3[i+1]; j++) {
+                          p_den[j] = p_den[j-1]*o_den[r+j];
+                          p_num[j] = p_num[j-1]*o_num[r+j];
                         }
                       }
+                      
+                      m += mi;
+                      if (mi > 0) v++;
                     }
                     
                     // unstabilized and stabilized weights
@@ -640,13 +754,13 @@ List msmcpp(
                       // truncated unstabilized weights
                       if (trunc_upper_only) {
                         double upper = quantilecpp(w, 1-trunc);
-                        for (i=0; i<n2; i++) {
+                        for (i=0; i<n3; i++) {
                           if (w[i] > upper) w[i] = upper;
                         }
                       } else {
                         double lower = quantilecpp(w, trunc);
                         double upper = quantilecpp(w, 1-trunc);
-                        for (i=0; i<n2; i++) {
+                        for (i=0; i<n3; i++) {
                           if (w[i] < lower) {
                             w[i] = lower;
                           } else if (w[i] > upper) {
@@ -658,13 +772,13 @@ List msmcpp(
                       // truncated stabilized weights
                       if (trunc_upper_only) {
                         double upper = quantilecpp(sw, 1-trunc);
-                        for (i=0; i<n2; i++) {
+                        for (i=0; i<n3; i++) {
                           if (sw[i] > upper) sw[i] = upper;
                         }
                       } else {
                         double lower = quantilecpp(sw, trunc);
                         double upper = quantilecpp(sw, 1-trunc);
-                        for (i=0; i<n2; i++) {
+                        for (i=0; i<n3; i++) {
                           if (sw[i] < lower) {
                             sw[i] = lower;
                           } else if (sw[i] > upper) {
@@ -712,31 +826,20 @@ List msmcpp(
                       fit_switch[h] = fit_x;
                     }
                     
-                    w1[l] = w;
-                    sw1[l] = sw;
+                    wb[l] = w;
+                    swb[l] = sw;
                   }
                   
                   // set up time-dependent treatment switching indicators
                   IntegerVector crossb(n);
                   for (i=0; i<n; i++) {
-                    if (swtrtb[i] == 1 && tstopb[i] >= swtrt_timeb[i]) {
+                    if (swtrtb[i] == 1 && tstartb[i] >= swtrt_timeb[i]) {
                       crossb[i] = 1;
                     } else {
                       crossb[i] = 0;
                     }
                   }
                   
-                  // weights do not change after treatment switch
-                  NumericVector wb(n, NA_REAL), swb(n, NA_REAL);
-                  wb[l] = w1;
-                  swb[l] = sw1;
-                  for (i=0; i<n; i++) {
-                    if (std::isnan(wb[i])) {
-                      wb[i] = wb[i-1];
-                      swb[i] = swb[i-1];
-                    }
-                  }
-                  
                   // prepare data for the outcome model
                   data_outcome = DataFrame::create(
                     Named("uid") = idb,
@@ -812,7 +915,7 @@ List msmcpp(
                   return out;
                 };
   
-  List out = f(idn, stratumn, tstartn, tstopn, eventn, treatn,
+  List out = f(idn, stratumn, tstartn, tstopn, eventn, treatn, os_timen,
                swtrtn, swtrt_timen, zn, zn_lgs_den);
   
   List data_switch = out["data_switch"];
@@ -866,71 +969,186 @@ List msmcpp(
   String hr_CI_type;
   NumericVector hrhats(n_boot);
   LogicalVector fails(n_boot);
+  DataFrame fail_boots_data;
   if (!boot) { // use Cox model to construct CI for HR if no boot
     hr_CI_type = "Cox model";
   } else { // bootstrap the entire process to construct CI for HR
     if (seed != NA_INTEGER) set_seed(seed);
     
-    int n0 = sum(treatn1==0);
-    int n1 = sum(treatn1==1);
     IntegerVector nobs = diff(idx);
     int N = max(nobs)*nids;
     
+    int B = N*n_boot;
+    IntegerVector boot_indexc(B);
+    IntegerVector oidc(B);
+    IntegerVector idc(B), stratumc(B), eventc(B), treatc(B);
+    IntegerVector osc(B), swtrtc(B);
+    NumericVector tstartc(B), tstopc(B), os_timec(B), swtrt_timec(B);
+    NumericMatrix zc_lgs_den(B, q+p2);
+    int index1 = 0;
+    
+    IntegerVector tsx(1,0); // first id within each treat/stratum
+    for (i=1; i<nids; i++) {
+      if ((treatn1[i] != treatn1[i-1]) || 
+          ((treatn1[i] == treatn1[i-1]) && 
+          (stratumn1[i] != stratumn1[i-1]))) {
+        tsx.push_back(i);
+      }
+    }
+    
+    int ntss = static_cast<int>(tsx.size());
+    tsx.push_back(nids);
+    
     for (k=0; k<n_boot; k++) {
-      IntegerVector idb(N), stratumb(N), treatb(N), eventb(N), swtrtb(N);
-      NumericVector tstartb(N), tstopb(N), swtrt_timeb(N);
+      IntegerVector oidb(N);
+      IntegerVector idb(N), stratumb(N), eventb(N), treatb(N);
+      IntegerVector osb(N), swtrtb(N);
+      NumericVector tstartb(N), tstopb(N), os_timeb(N), swtrt_timeb(N);
       NumericMatrix zb(N, p), zb_lgs_den(N, q+p2);
       
-      // sample the subject-level data with replacement by treatment group
+      // sample the subject-level data with replacement by treat/stratum
       int l = 0;
-      for (int h=0; h<nids; h++) {
-        double u = R::runif(0,1);
-        if (h < n0) {
-          i = static_cast<int>(std::floor(u*n0));
-        } else {
-          i = n0 + static_cast<int>(std::floor(u*n1));
-        }
-        
-        // create unique ids for bootstrap data sets
-        int idb1 = idn[idx[i]] + h*nids;
-        
-        for (j=idx[i]; j<idx[i+1]; j++) {
-          idb[l] = idb1;
-          stratumb[l] = stratumn[j];
-          tstartb[l] = tstartn[j];
-          tstopb[l] = tstopn[j];
-          eventb[l] = eventn[j];
-          treatb[l] = treatn[j];
-          swtrtb[l] = swtrtn[j];
-          swtrt_timeb[l] = swtrt_timen[j];
-          zb(l,_) = zn(j,_);
-          zb_lgs_den(l,_) = zn_lgs_den(j,_);
-          l++;
+      for (int h=0; h<ntss; h++) {
+        for (int r=tsx[h]; r<tsx[h+1]; r++) {
+          double u = R::runif(0,1);
+          i = tsx[h] + static_cast<int>(std::floor(u*(tsx[h+1]-tsx[h])));
+          
+          // create unique ids for bootstrap data sets
+          int oidb1 = idn[idx[i]];
+          int idb1 = idn[idx[i]] + r*nids;
+          
+          for (j=idx[i]; j<idx[i+1]; j++) {
+            oidb[l] = oidb1;
+            idb[l] = idb1;
+            stratumb[l] = stratumn[j];
+            tstartb[l] = tstartn[j];
+            tstopb[l] = tstopn[j];
+            eventb[l] = eventn[j];
+            treatb[l] = treatn[j];
+            osb[l] = osn[j];
+            os_timeb[l] = os_timen[j];
+            swtrtb[l] = swtrtn[j];
+            swtrt_timeb[l] = swtrt_timen[j];
+            zb(l,_) = zn(j,_);
+            zb_lgs_den(l,_) = zn_lgs_den(j,_);
+            l++;
+          }
         }
       }
       
       IntegerVector sub = Range(0,l-1);
+      oidb = oidb[sub];
       idb = idb[sub];
       stratumb = stratumb[sub];
       tstartb = tstartb[sub];
       tstopb = tstopb[sub];
       eventb = eventb[sub];
       treatb = treatb[sub];
+      osb = osb[sub];
+      os_timeb = os_timeb[sub];
       swtrtb = swtrtb[sub];
       swtrt_timeb = swtrt_timeb[sub];
       zb = subset_matrix_by_row(zb, sub);
       zb_lgs_den = subset_matrix_by_row(zb_lgs_den, sub);
       
-      out = f(idb, stratumb, tstartb, tstopb, eventb, treatb,
+      out = f(idb, stratumb, tstartb, tstopb, eventb, treatb, os_timeb,
               swtrtb, swtrt_timeb, zb, zb_lgs_den);
       
       fails[k] = out["fail"];
       hrhats[k] = out["hrhat"];
+      
+      if (fails[k]) {
+        for (i=0; i<l; i++) {
+          j = index1 + i;
+          boot_indexc[j] = k+1;
+          oidc[j] = oidb[i];
+          idc[j] = idb[i];
+          stratumc[j] = stratumb[i];
+          tstartc[j] = tstartb[i];
+          tstopc[j] = tstopb[i];
+          eventc[j] = eventb[i];
+          treatc[j] = treatb[i];
+          osc[j] = osb[i];
+          os_timec[j] = os_timeb[i];
+          swtrtc[j] = swtrtb[i];
+          swtrt_timec[j] = swtrt_timeb[i];
+          zc_lgs_den(j,_) = zb_lgs_den(i,_);
+        }
+        index1 += l;
+      }
+    }
+    
+    if (is_true(any(fails))) {
+      IntegerVector sub = seq(0,index1-1);
+      boot_indexc = boot_indexc[sub];
+      oidc = oidc[sub];
+      idc = idc[sub];
+      stratumc = stratumc[sub];
+      tstartc = tstartc[sub];
+      tstopc = tstopc[sub];
+      eventc = eventc[sub];
+      treatc = treatc[sub];
+      osc = osc[sub];
+      os_timec = os_timec[sub];
+      swtrtc = swtrtc[sub];
+      swtrt_timec = swtrt_timec[sub];
+      zc_lgs_den = subset_matrix_by_row(zc_lgs_den,sub);
+      
+      fail_boots_data = DataFrame::create(
+        Named("boot_index") = boot_indexc,
+        Named("uid") = idc,
+        Named("tstart") = tstartc,
+        Named("tstop") = tstopc,
+        Named("event") = eventc,
+        Named("treated") = treatc,
+        Named("os") = osc,
+        Named("os_time") = os_timec,
+        Named("swtrt") = swtrtc,
+        Named("swtrt_time") = swtrt_timec
+      );
+      
+      for (j=0; j<q+p2; j++) {
+        String zj = covariates_lgs_den[j];
+        NumericVector u = zc_lgs_den(_,j);
+        fail_boots_data.push_back(u, zj);
+      }
+      
+      if (TYPEOF(data[id]) == INTSXP) {
+        fail_boots_data.push_back(idwi[oidc-1], id);
+      } else if (TYPEOF(data[id]) == REALSXP) {
+        fail_boots_data.push_back(idwn[oidc-1], id);
+      } else if (TYPEOF(data[id]) == STRSXP) {
+        fail_boots_data.push_back(idwc[oidc-1], id);
+      }
+      
+      if (TYPEOF(data[treat]) == LGLSXP || TYPEOF(data[treat]) == INTSXP) {
+        fail_boots_data.push_back(treatwi[1-treatc], treat);
+      } else if (TYPEOF(data[treat]) == REALSXP) {
+        fail_boots_data.push_back(treatwn[1-treatc], treat);
+      } else if (TYPEOF(data[treat]) == STRSXP) {
+        fail_boots_data.push_back(treatwc[1-treatc], treat);
+      }
+      
+      if (has_stratum) {
+        for (i=0; i<p_stratum; i++) {
+          String s = stratum[i];
+          if (TYPEOF(data[s]) == INTSXP) {
+            IntegerVector stratumwi = u_stratum[s];
+            fail_boots_data.push_back(stratumwi[stratumc-1], s);
+          } else if (TYPEOF(data[s]) == REALSXP) {
+            NumericVector stratumwn = u_stratum[s];
+            fail_boots_data.push_back(stratumwn[stratumc-1], s);
+          } else if (TYPEOF(data[s]) == STRSXP) {
+            StringVector stratumwc = u_stratum[s];
+            fail_boots_data.push_back(stratumwc[stratumc-1], s);
+          }
+        }
+      }
     }
     
     // obtain bootstrap confidence interval for HR
     double loghr = log(hrhat);
-    LogicalVector ok = 1 - fails;
+    LogicalVector ok = (1 - fails) & !is_na(hrhats);
     int n_ok = sum(ok);
     NumericVector loghrs = log(hrhats[ok]);
     double sdloghr = sd(loghrs);
@@ -973,6 +1191,9 @@ List msmcpp(
   if (boot) {
     result.push_back(fails, "fail_boots");
     result.push_back(hrhats, "hr_boots"); 
+    if (is_true(any(fails))) {
+      result.push_back(fail_boots_data, "fail_boots_data");
+    }
   }
   
   return result;
