@@ -26,12 +26,12 @@ double est_psi_rpsftm(
     const bool autoswitch,
     const double alpha,
     const std::string ties) {
-
+  
   NumericVector init(1, NA_REAL);
   
   DataFrame Sstar = untreated(psi*treat_modifier, id, time, event, treat, 
                               rx, censor_time, recensor, autoswitch);
-
+  
   double z = NA_REAL;
   if (test == "logrank") {
     Sstar.push_back(stratum, "ustratum");
@@ -101,11 +101,11 @@ List rpsftmcpp(const DataFrame data,
                const bool boot = 0,
                const int n_boot = 1000,
                const int seed = NA_INTEGER) {
-
+  
   int i, j, k, n = data.nrow();
   int p = static_cast<int>(base_cov.size());
   if (p == 1 && (base_cov[0] == "" || base_cov[0] == "none")) p = 0;
-
+  
   int p_stratum = static_cast<int>(stratum.size());
   
   bool has_stratum;
@@ -113,11 +113,13 @@ List rpsftmcpp(const DataFrame data,
   DataFrame u_stratum;
   IntegerVector d(p_stratum);
   IntegerMatrix stratan(n,p_stratum);
+  List levels(p_stratum);
   if (p_stratum == 1 && (stratum[0] == "" || stratum[0] == "none")) {
     has_stratum = 0;
     stratumn.fill(1);
     d[0] = 1;
     stratan(_,0) = stratumn;
+    levels[0] = 1;
   } else {
     List out = bygroup(data, stratum);
     has_stratum = 1;
@@ -125,6 +127,7 @@ List rpsftmcpp(const DataFrame data,
     u_stratum = DataFrame(out["lookup"]);
     d = out["nlevels"];
     stratan = as<IntegerMatrix>(out["indices"]);
+    levels = out["lookups"];
   }
   
   IntegerVector stratumn_unique = unique(stratumn);
@@ -136,7 +139,7 @@ List rpsftmcpp(const DataFrame data,
   bool has_treat = hasVariable(data, treat);
   bool has_rx = hasVariable(data, rx);
   bool has_censor_time = hasVariable(data, censor_time);
-
+  
   if (!has_id) {
     stop("data must contain the id variable");
   }
@@ -167,39 +170,49 @@ List rpsftmcpp(const DataFrame data,
   if (!has_time) {
     stop("data must contain the time variable");
   }
-
+  
   if (TYPEOF(data[time]) != INTSXP && TYPEOF(data[time]) != REALSXP) {
     stop("time must take numeric values");
   }
-
+  
   NumericVector timenz = data[time];
   NumericVector timen = clone(timenz);
   if (is_true(any(timen < 0.0))) {
     stop("time must be nonnegative");
   }
-
+  
   if (!has_event) {
     stop("data must contain the event variable");
   }
-
-  if (TYPEOF(data[event]) != INTSXP && TYPEOF(data[event]) != LGLSXP) {
-    stop("event must take integer or logical values");
+  
+  IntegerVector eventn(n);
+  if (TYPEOF(data[event]) == LGLSXP || TYPEOF(data[event]) == INTSXP) {
+    IntegerVector eventnz = data[event];
+    if (is_true(any((eventnz != 1) & (eventnz != 0)))) {
+      stop("event must be 1 or 0 for each subject");
+    } else {
+      eventn = clone(eventnz);
+    }
+  } else if (TYPEOF(data[event]) == REALSXP) {
+    NumericVector eventnz = data[event];
+    if (is_true(any((eventnz != 1) & (eventnz != 0)))) {
+      stop("event must be 1 or 0 for each subject");
+    } else {
+      NumericVector eventnz2 = clone(eventnz);
+      eventn = as<IntegerVector>(eventnz2);
+    }
+  } else {
+    stop("event must take logical, integer, or real values");
   }
-
-  IntegerVector eventnz = data[event];
-  IntegerVector eventn = clone(eventnz);
-  if (is_true(any((eventn != 1) & (eventn != 0)))) {
-    stop("event must be 1 or 0");
-  }
-
+  
   if (is_true(all(eventn == 0))) {
     stop("at least 1 event is needed");
   }
-
+  
   if (!has_treat) {
     stop("data must contain the treat variable");
   }
-
+  
   // create the numeric treat variable
   IntegerVector treatn(n);
   IntegerVector treatwi;
@@ -250,32 +263,32 @@ List rpsftmcpp(const DataFrame data,
   if (!has_rx) {
     stop("data must contain the rx variable");
   }
-
+  
   if (TYPEOF(data[rx]) != INTSXP && TYPEOF(data[rx]) != REALSXP) {
     stop("rx must take numeric values");
   }
-
+  
   NumericVector rxnz = data[rx];
   NumericVector rxn = clone(rxnz);
   if (is_true(any((rxn < 0.0) | (rxn > 1.0)))) {
     stop("rx must take values between 0 and 1");
   }
-
+  
   if (!has_censor_time) {
     stop("data must contain the censor_time variable");
   }
-
+  
   if (TYPEOF(data[censor_time]) != INTSXP &&
       TYPEOF(data[censor_time]) != REALSXP) {
     stop("censor_time must take numeric values");
   }
-
+  
   NumericVector censor_timenz = data[censor_time];
   NumericVector censor_timen = clone(censor_timenz);
   if (is_true(any(censor_timen < timen))) {
     stop("censor_time must be greater than or equal to time");
   }
-
+  
   if (!admin_recensor_only) {
     for (i=0; i<n; i++) {
       if (eventn[i] == 0) { // use the actual censoring time for dropouts
@@ -283,7 +296,7 @@ List rpsftmcpp(const DataFrame data,
       }
     }
   }
-
+  
   // covariates for the Cox proportional hazards model
   // containing treat and base_cov
   StringVector covariates(p+1);
@@ -301,7 +314,7 @@ List rpsftmcpp(const DataFrame data,
     covariates[j+1] = zj;
     zn(_,j) = u;
   }
-
+  
   // covariates for the accelerated failure time model
   // including treat, stratum, and base_cov
   int q; // number of columns corresponding to the strata effects
@@ -316,18 +329,55 @@ List rpsftmcpp(const DataFrame data,
   covariates_aft[0] = "treated";
   if (strata_main_effect_only) {
     k = 0;
-    for (i=0; i<p_stratum; i++) {
-      for (j=0; j<d[i]-1; j++) {
-        covariates_aft[k+j+1] = "stratum_" + std::to_string(i+1) +
-          "_level_" + std::to_string(j+1);
-        zn_aft(_,k+j) = 1.0*(stratan(_,i) == j+1);
+    for (i=0; i<p_stratum; ++i) {
+      int di = d[i]-1;
+      for (j=0; j<di; ++j) {
+        covariates_aft[k+j+1] = as<std::string>(stratum[i]);
+        if (TYPEOF(levels[i]) == STRSXP) {
+          StringVector u = levels[i];
+          std::string label = sanitize(as<std::string>(u[j]));
+          covariates_aft[k+j+1] += label;
+        } else if (TYPEOF(levels[i]) == REALSXP) {
+          NumericVector u = levels[i];
+          covariates_aft[k+j+1] += std::to_string(u[j]);
+        } else if (TYPEOF(levels[i]) == INTSXP 
+                     || TYPEOF(levels[i]) == LGLSXP) {
+          IntegerVector u = levels[i];
+          covariates_aft[k+j+1] += std::to_string(u[j]);
+        }
+        zn_aft(_,k+j) = (stratan(_,i) == j+1);
       }
-      k += d[i]-1;
+      k += di;
     }
   } else {
-    for (j=0; j<nstrata-1; j++) {
-      covariates_aft[j+1] = "stratum_" + std::to_string(j+1);
-      zn_aft(_,j) = 1.0*(stratumn == j+1);
+    for (j=0; j<nstrata-1; ++j) {
+      // locate the first observation in the stratum
+      int first_k = 0;
+      for (; first_k<n; ++first_k) {
+        if (stratumn[first_k] == j+1) break;
+      }
+      covariates_aft[j+1] = "";
+      for (i=0; i<p_stratum; ++i) {
+        IntegerVector q_col = stratan(_,i);
+        int l = q_col[first_k] - 1;
+        covariates_aft[j+1] += as<std::string>(stratum[i]);
+        if (TYPEOF(levels[i]) == STRSXP) {
+          StringVector u = levels[i];
+          std::string label = sanitize(as<std::string>(u[l]));
+          covariates_aft[j+1] += label;
+        } else if (TYPEOF(levels[i]) == REALSXP) {
+          NumericVector u = levels[i];
+          covariates_aft[j+1] += std::to_string(u[l]);
+        } else if (TYPEOF(levels[i]) == INTSXP 
+                     || TYPEOF(levels[i]) == LGLSXP) {
+          IntegerVector u = levels[i];
+          covariates_aft[j+1] += std::to_string(u[l]);
+        }
+        if (i < p_stratum-1) {
+          covariates_aft[j+1] += ".";
+        }
+      }
+      zn_aft(_,j) = (stratumn == j+1);
     }
   }
   
@@ -374,15 +424,15 @@ List rpsftmcpp(const DataFrame data,
   if (low_psi >= hi_psi) {
     stop("low_psi must be less than hi_psi");
   }
-
+  
   if (n_eval_z < 2) {
     stop("n_eval_z must be greater than or equal to 2");
   }
-
+  
   if (treat_modifier <= 0.0) {
     stop("treat_modifier must be positive");
   }
-
+  
   if (alpha <= 0.0 || alpha >= 0.5) {
     stop("alpha must lie between 0 and 0.5");
   }
@@ -398,11 +448,11 @@ List rpsftmcpp(const DataFrame data,
   if (n_boot < 100) {
     stop("n_boot must be greater than or equal to 100");
   }
-
+  
   
   DataFrame lr = lrtest(data, "", stratum, treat, time, event, 0, 0);
   double logRankPValue = lr["logRankPValue"];
-
+  
   // evaluate the log-rank test statistic at each psi
   double step_psi = (hi_psi - low_psi)/(n_eval_z - 1);
   NumericVector psi(n_eval_z), Z(n_eval_z);
@@ -413,13 +463,13 @@ List rpsftmcpp(const DataFrame data,
                           covariates_aft, zn_aft, dist, treat_modifier, 
                           recensor, autoswitch, alpha, ties);
   }
-
+  
   DataFrame eval_z = DataFrame::create(
     Named("psi") = psi,
     Named("Z") = Z);
-
+  
   double zcrit = R::qnorm(1-alpha/2, 0, 1, 1, 0);
-
+  
   k = -1;
   auto f = [&k, n, q, p, test, covariates, covariates_aft, dist, low_psi, 
             hi_psi, n_eval_z, psi, treat_modifier, recensor, autoswitch, 
@@ -434,9 +484,10 @@ List rpsftmcpp(const DataFrame data,
                   NumericVector init(1, NA_REAL);
                   
                   // obtain the estimate and confidence interval of psi
-                  double psihat, psilower = 0, psiupper = 0;
+                  double psihat = NA_REAL;
+                  double psilower = NA_REAL, psiupper = NA_REAL;
                   String psi_CI_type;
-
+                  
                   if (gridsearch) {
                     NumericVector Z(n_eval_z);
                     for (i=0; i<n_eval_z; i++) {
@@ -446,19 +497,13 @@ List rpsftmcpp(const DataFrame data,
                         covariates_aft, zb_aft, dist, treat_modifier, 
                         recensor, autoswitch, alpha, ties);
                     }
-
-                    auto g = [psi, Z](double target)->double{
-                      NumericVector Z1 = Z - target;
-                      NumericVector Zsq = Z1*Z1;
-                      return psi[which_min(Zsq)];
-                    };
-
-                    psihat = g(0);
+                    
+                    psihat = getpsiest(0, psi, Z);
                     psi_CI_type = "grid search";
-
+                    
                     if (k == -1) {
-                      psilower = g(zcrit);
-                      psiupper = g(-zcrit);
+                      psilower = getpsiest(zcrit, psi, Z);
+                      psiupper = getpsiest(-zcrit, psi, Z);
                     }
                   } else {
                     double target = 0;
@@ -475,73 +520,92 @@ List rpsftmcpp(const DataFrame data,
                                   autoswitch, alpha, ties);
                                 return z - target;
                               };
-
-                    psihat = brent(g, low_psi, hi_psi, tol);
+                    
+                    double psilo = getpsiend(g, 1, low_psi);
+                    double psihi = getpsiend(g, 0, hi_psi);
+                    if (!std::isnan(psilo) && !std::isnan(psihi)) {
+                      psihat = brent(g, psilo, psihi, tol);
+                    }
                     psi_CI_type = "root finding";
-
+                    
                     if (k == -1) {
                       target = zcrit;
-                      if (g(low_psi) > 0) {
-                        psilower = brent(g, low_psi, psihat, tol);  
-                      } else {
-                        psilower = NA_REAL;
+                      psilo = getpsiend(g, 1, low_psi);
+                      psihi = getpsiend(g, 0, hi_psi);
+                      if (!std::isnan(psilo) && !std::isnan(psihi)) {
+                        if (!std::isnan(psihat)) {
+                          psilower = brent(g, psilo, psihat, tol);
+                        } else {
+                          psilower = brent(g, psilo, psihi, tol);
+                        }
                       }
                       
                       target = -zcrit;
-                      if (g(hi_psi) < 0) {
-                        psiupper = brent(g, psihat, hi_psi, tol);  
-                      } else {
-                        psiupper = NA_REAL;
+                      psilo = getpsiend(g, 1, low_psi);
+                      psihi = getpsiend(g, 0, hi_psi);
+                      if (!std::isnan(psilo) && !std::isnan(psihi)) {
+                        if (!std::isnan(psihat)) {
+                          psiupper = brent(g, psihat, psihi, tol);  
+                        } else {
+                          psiupper = brent(g, psilo, psihi, tol);  
+                        }
                       }
                     }
                   }
-
-                  // obtain the Kaplan-Meier estimates
-                  DataFrame Sstar, kmstar;
-                  if (k == -1) {
-                    Sstar = untreated(
-                      psihat*treat_modifier, idb, timeb, eventb, treatb,
-                      rxb, censor_timeb, recensor, autoswitch);
-
-                    kmstar = kmest(Sstar, "", "treated", "t_star",
-                                   "d_star", "log-log", 1-alpha, 1);
+                  
+                  DataFrame Sstar, kmstar, data_outcome;
+                  List fit_outcome;
+                  double hrhat = NA_REAL, pvalue = NA_REAL;
+                  
+                  bool psimissing = std::isnan(psihat);
+                  
+                  if (!psimissing) {
+                    // obtain the Kaplan-Meier estimates
+                    if (k == -1) {
+                      Sstar = untreated(
+                        psihat*treat_modifier, idb, timeb, eventb, treatb,
+                        rxb, censor_timeb, recensor, autoswitch);
+                      
+                      kmstar = kmest(Sstar, "", "treated", "t_star",
+                                     "d_star", "log-log", 1-alpha, 1);
+                      
+                      Sstar.push_back(stratumb, "ustratum");
+                      
+                      for (j=0; j<p; j++) {
+                        String zj = covariates[j+1];
+                        NumericVector u = zb(_,j);
+                        Sstar.push_back(u, zj);
+                      }
+                    }
                     
-                    Sstar.push_back(stratumb, "ustratum");
+                    // run Cox model to obtain the hazard ratio estimate
+                    data_outcome = unswitched(
+                      psihat*treat_modifier, n, idb, timeb, eventb, treatb,
+                      rxb, censor_timeb, recensor, autoswitch);
+                    
+                    data_outcome.push_back(stratumb, "ustratum");
                     
                     for (j=0; j<p; j++) {
                       String zj = covariates[j+1];
                       NumericVector u = zb(_,j);
-                      Sstar.push_back(u, zj);
+                      data_outcome.push_back(u, zj);
                     }
+                    
+                    fit_outcome = phregcpp(
+                      data_outcome, "", "ustratum", "t_star", "", "d_star", 
+                      covariates, "", "", "", ties, init, 
+                      0, 0, 0, 0, 0, alpha, 50, 1.0e-9);
+                    
+                    DataFrame sumstat_cox = DataFrame(fit_outcome["sumstat"]);
+                    bool fail_cox = sumstat_cox["fail"];
+                    if (fail_cox == 1) fail = 1;
+                    
+                    DataFrame parest = DataFrame(fit_outcome["parest"]);
+                    NumericVector beta = parest["beta"];
+                    NumericVector pval = parest["p"];
+                    hrhat = exp(beta[0]);
+                    pvalue = pval[0];                    
                   }
-
-                  // run Cox model to obtain the hazard ratio estimate
-                  DataFrame data_outcome = unswitched(
-                    psihat*treat_modifier, n, idb, timeb, eventb, treatb,
-                    rxb, censor_timeb, recensor, autoswitch);
-
-                  data_outcome.push_back(stratumb, "ustratum");
-                  
-                  for (j=0; j<p; j++) {
-                    String zj = covariates[j+1];
-                    NumericVector u = zb(_,j);
-                    data_outcome.push_back(u, zj);
-                  }
-
-                  List fit_outcome = phregcpp(
-                    data_outcome, "", "ustratum", "t_star", "", "d_star", 
-                    covariates, "", "", "", ties, init, 
-                    0, 0, 0, 0, 0, alpha, 50, 1.0e-9);
-
-                  DataFrame sumstat_cox = DataFrame(fit_outcome["sumstat"]);
-                  bool fail_cox = sumstat_cox["fail"];
-                  if (fail_cox == 1) fail = 1;
-                  
-                  DataFrame parest = DataFrame(fit_outcome["parest"]);
-                  NumericVector beta = parest["beta"];
-                  NumericVector pval = parest["p"];
-                  double hrhat = exp(beta[0]);
-                  double pvalue = pval[0];
                   
                   List out;
                   if (k == -1) {
@@ -556,106 +620,28 @@ List rpsftmcpp(const DataFrame data,
                       Named("psi_CI_type") = psi_CI_type,
                       Named("hrhat") = hrhat,
                       Named("pvalue") = pvalue,
-                      Named("fail") = fail);
+                      Named("fail") = fail,
+                      Named("psimissing") = psimissing);
                   } else {
                     out = List::create(
                       Named("psihat") = psihat,
                       Named("hrhat") = hrhat,
                       Named("pvalue") = pvalue,
-                      Named("fail") = fail);
+                      Named("fail") = fail,
+                      Named("psimissing") = psimissing);
                   }
-
+                  
                   return out;
                 };
-
+  
   List out = f(idn, stratumn, timen, eventn, treatn, rxn, censor_timen, 
                zn, zn_aft);
-
+  
   DataFrame Sstar = DataFrame(out["Sstar"]);
   DataFrame kmstar = DataFrame(out["kmstar"]);
   DataFrame data_outcome = DataFrame(out["data_outcome"]);
   List fit_outcome = out["fit_outcome"];
   
-  IntegerVector uid = Sstar["uid"];
-  if (TYPEOF(data[id]) == INTSXP) {
-    Sstar.push_front(idwi[uid-1], id);
-  } else if (TYPEOF(data[id]) == REALSXP) {
-    Sstar.push_front(idwn[uid-1], id);
-  } else if (TYPEOF(data[id]) == STRSXP) {
-    Sstar.push_front(idwc[uid-1], id);
-  }
-  
-  uid = data_outcome["uid"];
-  if (TYPEOF(data[id]) == INTSXP) {
-    data_outcome.push_front(idwi[uid-1], id);
-  } else if (TYPEOF(data[id]) == REALSXP) {
-    data_outcome.push_front(idwn[uid-1], id);
-  } else if (TYPEOF(data[id]) == STRSXP) {
-    data_outcome.push_front(idwc[uid-1], id);
-  }
-  
-  
-  IntegerVector treated = Sstar["treated"];
-  if (TYPEOF(data[treat]) == LGLSXP || TYPEOF(data[treat]) == INTSXP) {
-    Sstar.push_back(treatwi[1-treated], treat);
-  } else if (TYPEOF(data[treat]) == REALSXP) {
-    Sstar.push_back(treatwn[1-treated], treat);
-  } else if (TYPEOF(data[treat]) == STRSXP) {
-    Sstar.push_back(treatwc[1-treated], treat);
-  }
-  
-  treated = kmstar["treated"];
-  if (TYPEOF(data[treat]) == LGLSXP || TYPEOF(data[treat]) == INTSXP) {
-    kmstar.push_back(treatwi[1-treated], treat);
-  } else if (TYPEOF(data[treat]) == REALSXP) {
-    kmstar.push_back(treatwn[1-treated], treat);
-  } else if (TYPEOF(data[treat]) == STRSXP) {
-    kmstar.push_back(treatwc[1-treated], treat);
-  }
-  
-  treated = data_outcome["treated"];
-  if (TYPEOF(data[treat]) == LGLSXP || TYPEOF(data[treat]) == INTSXP) {
-    data_outcome.push_back(treatwi[1-treated], treat);
-  } else if (TYPEOF(data[treat]) == REALSXP) {
-    data_outcome.push_back(treatwn[1-treated], treat);
-  } else if (TYPEOF(data[treat]) == STRSXP) {
-    data_outcome.push_back(treatwc[1-treated], treat);
-  }
-  
-  
-  if (has_stratum) {
-    IntegerVector ustratum = Sstar["ustratum"];
-    for (i=0; i<p_stratum; i++) {
-      String s = stratum[i];
-      if (TYPEOF(data[s]) == INTSXP) {
-        IntegerVector stratumwi = u_stratum[s];
-        Sstar.push_back(stratumwi[ustratum-1], s);
-      } else if (TYPEOF(data[s]) == REALSXP) {
-        NumericVector stratumwn = u_stratum[s];
-        Sstar.push_back(stratumwn[ustratum-1], s);
-      } else if (TYPEOF(data[s]) == STRSXP) {
-        StringVector stratumwc = u_stratum[s];
-        Sstar.push_back(stratumwc[ustratum-1], s);
-      }
-    }
-    
-    ustratum = data_outcome["ustratum"];
-    for (i=0; i<p_stratum; i++) {
-      String s = stratum[i];
-      if (TYPEOF(data[s]) == INTSXP) {
-        IntegerVector stratumwi = u_stratum[s];
-        data_outcome.push_back(stratumwi[ustratum-1], s);
-      } else if (TYPEOF(data[s]) == REALSXP) {
-        NumericVector stratumwn = u_stratum[s];
-        data_outcome.push_back(stratumwn[ustratum-1], s);
-      } else if (TYPEOF(data[s]) == STRSXP) {
-        StringVector stratumwc = u_stratum[s];
-        data_outcome.push_back(stratumwc[ustratum-1], s);
-      }
-    }
-  }
-  
-  
   double psihat = out["psihat"];
   double psilower = out["psilower"];
   double psiupper = out["psiupper"];
@@ -663,98 +649,275 @@ List rpsftmcpp(const DataFrame data,
   double hrhat = out["hrhat"];
   double pvalue = out["pvalue"];
   bool fail = out["fail"];
+  bool psimissing = out["psimissing"];
   
-  // construct the confidence interval for HR
-  double hrlower, hrupper;
+  double hrlower = NA_REAL, hrupper = NA_REAL;
   NumericVector hrhats(n_boot), psihats(n_boot);
   LogicalVector fails(n_boot);
+  DataFrame fail_boots_data;
   String hr_CI_type;
-  if (!boot) { // use log-rank p-value to construct CI for HR if no boot
-    double loghr = log(hrhat);
-    double zcox = R::qnorm(logRankPValue, 0, 1, 1, 0);
-    double seloghr = loghr/zcox;
-    hrlower = exp(loghr - zcrit*seloghr);
-    hrupper = exp(loghr + zcrit*seloghr);
-    hr_CI_type = "log-rank p-value";
-  } else { // bootstrap the entire process to construct CI for HR
-    if (seed != NA_INTEGER) set_seed(seed);
-
-    IntegerVector idb(n), stratumb(n), treatb(n), eventb(n);
-    NumericVector timeb(n), rxb(n), censor_timeb(n);
-    NumericMatrix zb(n,p), zb_aft(n,q+p);
-
-    // sort data by treatment group
-    IntegerVector idx0 = which(treatn == 0);
-    IntegerVector idx1 = which(treatn == 1);
-    int n0 = static_cast<int>(idx0.size());
-    int n1 = static_cast<int>(idx1.size());
-    IntegerVector order(n);
-    for (i=0; i<n0; i++) {
-      order[i] = idx0[i];
+  
+  if (!psimissing) {
+    IntegerVector uid = Sstar["uid"];
+    if (TYPEOF(data[id]) == INTSXP) {
+      Sstar.push_front(idwi[uid-1], id);
+    } else if (TYPEOF(data[id]) == REALSXP) {
+      Sstar.push_front(idwn[uid-1], id);
+    } else if (TYPEOF(data[id]) == STRSXP) {
+      Sstar.push_front(idwc[uid-1], id);
     }
-    for (i=0; i<n1; i++){
-      order[n0+i] = idx1[i];
-    }
-
-    idn = idn[order];
-    stratumn = stratumn[order];
-    timen = timen[order];
-    eventn = eventn[order];
-    treatn = treatn[order];
-    rxn = rxn[order];
-    censor_timen = censor_timen[order];
-    zn = subset_matrix_by_row(zn, order);
-    zn_aft = subset_matrix_by_row(zn_aft, order);
     
-    for (k=0; k<n_boot; k++) {
-      // sample the data with replacement by treatment group
-      for (i=0; i<n; i++) {
-        double u = R::runif(0,1);
-        if (treatn[i] == 0) {
-          j = static_cast<int>(std::floor(u*n0));
-        } else {
-          j = n0 + static_cast<int>(std::floor(u*n1));
+    uid = data_outcome["uid"];
+    if (TYPEOF(data[id]) == INTSXP) {
+      data_outcome.push_front(idwi[uid-1], id);
+    } else if (TYPEOF(data[id]) == REALSXP) {
+      data_outcome.push_front(idwn[uid-1], id);
+    } else if (TYPEOF(data[id]) == STRSXP) {
+      data_outcome.push_front(idwc[uid-1], id);
+    }
+    
+    IntegerVector treated = Sstar["treated"];
+    if (TYPEOF(data[treat]) == LGLSXP || TYPEOF(data[treat]) == INTSXP) {
+      Sstar.push_back(treatwi[1-treated], treat);
+    } else if (TYPEOF(data[treat]) == REALSXP) {
+      Sstar.push_back(treatwn[1-treated], treat);
+    } else if (TYPEOF(data[treat]) == STRSXP) {
+      Sstar.push_back(treatwc[1-treated], treat);
+    }
+    
+    treated = kmstar["treated"];
+    if (TYPEOF(data[treat]) == LGLSXP || TYPEOF(data[treat]) == INTSXP) {
+      kmstar.push_back(treatwi[1-treated], treat);
+    } else if (TYPEOF(data[treat]) == REALSXP) {
+      kmstar.push_back(treatwn[1-treated], treat);
+    } else if (TYPEOF(data[treat]) == STRSXP) {
+      kmstar.push_back(treatwc[1-treated], treat);
+    }
+    
+    treated = data_outcome["treated"];
+    if (TYPEOF(data[treat]) == LGLSXP || TYPEOF(data[treat]) == INTSXP) {
+      data_outcome.push_back(treatwi[1-treated], treat);
+    } else if (TYPEOF(data[treat]) == REALSXP) {
+      data_outcome.push_back(treatwn[1-treated], treat);
+    } else if (TYPEOF(data[treat]) == STRSXP) {
+      data_outcome.push_back(treatwc[1-treated], treat);
+    }
+    
+    if (has_stratum) {
+      IntegerVector ustratum = Sstar["ustratum"];
+      for (i=0; i<p_stratum; i++) {
+        String s = stratum[i];
+        if (TYPEOF(data[s]) == INTSXP) {
+          IntegerVector stratumwi = u_stratum[s];
+          Sstar.push_back(stratumwi[ustratum-1], s);
+        } else if (TYPEOF(data[s]) == REALSXP) {
+          NumericVector stratumwn = u_stratum[s];
+          Sstar.push_back(stratumwn[ustratum-1], s);
+        } else if (TYPEOF(data[s]) == STRSXP) {
+          StringVector stratumwc = u_stratum[s];
+          Sstar.push_back(stratumwc[ustratum-1], s);
         }
-
-        idb[i] = idn[j];
-        stratumb[i] = stratumn[j];
-        timeb[i] = timen[j];
-        eventb[i] = eventn[j];
-        treatb[i] = treatn[j];
-        rxb[i] = rxn[j];
-        censor_timeb[i] = censor_timen[j];
-        zb(i,_) = zn(j,_);
-        zb_aft(i,_) = zn_aft(j,_);
       }
-
-      List out = f(idb, stratumb, timeb, eventb, treatb, rxb, 
-                   censor_timeb, zb, zb_aft);
       
-      fails[k] = out["fail"];
-      hrhats[k] = out["hrhat"];
-      psihats[k] = out["psihat"];
+      ustratum = data_outcome["ustratum"];
+      for (i=0; i<p_stratum; i++) {
+        String s = stratum[i];
+        if (TYPEOF(data[s]) == INTSXP) {
+          IntegerVector stratumwi = u_stratum[s];
+          data_outcome.push_back(stratumwi[ustratum-1], s);
+        } else if (TYPEOF(data[s]) == REALSXP) {
+          NumericVector stratumwn = u_stratum[s];
+          data_outcome.push_back(stratumwn[ustratum-1], s);
+        } else if (TYPEOF(data[s]) == STRSXP) {
+          StringVector stratumwc = u_stratum[s];
+          data_outcome.push_back(stratumwc[ustratum-1], s);
+        }
+      }
     }
     
-    // obtain bootstrap confidence interval for HR
-    double loghr = log(hrhat);
-    LogicalVector ok = 1 - fails;
-    int n_ok = sum(ok);
-    NumericVector loghrs = log(hrhats[ok]);
-    double sdloghr = sd(loghrs);
-    double tcrit = R::qt(1-alpha/2, n_ok-1, 1, 0);
-    hrlower = exp(loghr - tcrit*sdloghr);
-    hrupper = exp(loghr + tcrit*sdloghr);
-    hr_CI_type = "bootstrap";
-    pvalue = 2*(1 - R::pt(fabs(loghr/sdloghr), n_ok-1, 1, 0));
-    
-    // obtain bootstrap confidence interval for psi
-    NumericVector psihats1 = psihats[ok];
-    double sdpsi = sd(psihats1);
-    psilower = psihat - tcrit*sdpsi;
-    psiupper = psihat + tcrit*sdpsi;
-    psi_CI_type = "bootstrap";
+    // construct the confidence interval for HR
+    if (!boot) { // use log-rank p-value to construct CI for HR if no boot
+      double loghr = log(hrhat);
+      double zcox = R::qnorm(logRankPValue, 0, 1, 1, 0);
+      double seloghr = loghr/zcox;
+      hrlower = exp(loghr - zcrit*seloghr);
+      hrupper = exp(loghr + zcrit*seloghr);
+      hr_CI_type = "log-rank p-value";
+    } else { // bootstrap the entire process to construct CI for HR
+      if (seed != NA_INTEGER) set_seed(seed);
+      
+      IntegerVector idb(n), stratumb(n), treatb(n), eventb(n);
+      NumericVector timeb(n), rxb(n), censor_timeb(n);
+      NumericMatrix zb(n,p), zb_aft(n,q+p);
+      
+      int B = n*n_boot;
+      IntegerVector boot_indexc(B);
+      IntegerVector idc(B), stratumc(B), treatc(B), eventc(B);
+      NumericVector timec(B), rxc(B), censor_timec(B);
+      NumericMatrix zc_aft(B,q+p);
+      int index1 = 0;
+      
+      // sort data by treatment group, stratum and id
+      IntegerVector order = seq(0, n-1);
+      if (has_stratum) {
+        std::sort(order.begin(), order.end(), [&](int i, int j) {
+          return ((treatn[i] < treatn[j]) ||
+                  ((treatn[i] == treatn[j]) && (stratumn[i] < stratumn[j])) || 
+                  ((treatn[i] == treatn[j]) && (stratumn[i] == stratumn[j]) &&
+                  (idn[i] < idn[j])));
+        });
+      } else {
+        std::sort(order.begin(), order.end(), [&](int i, int j) {
+          return ((treatn[i] < treatn[j]) ||
+                  ((treatn[i] == treatn[j]) && (idn[i] < idn[j])));
+        });
+      }
+      
+      idn = idn[order];
+      stratumn = stratumn[order];
+      timen = timen[order];
+      eventn = eventn[order];
+      treatn = treatn[order];
+      rxn = rxn[order];
+      censor_timen = censor_timen[order];
+      zn = subset_matrix_by_row(zn, order);
+      zn_aft = subset_matrix_by_row(zn_aft, order);
+      
+      IntegerVector tsx(1,0); // first observation within each treat/stratum
+      for (i=1; i<n; i++) {
+        if ((treatn[i] != treatn[i-1]) || 
+            ((treatn[i] == treatn[i-1]) && (stratumn[i] != stratumn[i-1]))) {
+          tsx.push_back(i);
+        }
+      }
+      
+      int ntss = static_cast<int>(tsx.size());
+      tsx.push_back(n);
+      
+      for (k=0; k<n_boot; k++) {
+        // sample the data with replacement by treatment group and stratum
+        for (int h=0; h<ntss; h++) {
+          for (i=tsx[h]; i<tsx[h+1]; i++) {
+            double u = R::runif(0,1);
+            j = tsx[h] + static_cast<int>(std::floor(u*(tsx[h+1]-tsx[h])));
+            
+            idb[i] = idn[j];
+            stratumb[i] = stratumn[j];
+            timeb[i] = timen[j];
+            eventb[i] = eventn[j];
+            treatb[i] = treatn[j];
+            rxb[i] = rxn[j];
+            censor_timeb[i] = censor_timen[j];
+            zb(i,_) = zn(j,_);
+            zb_aft(i,_) = zn_aft(j,_);
+          }
+        }
+        
+        List out = f(idb, stratumb, timeb, eventb, treatb, rxb, 
+                     censor_timeb, zb, zb_aft);
+        
+        fails[k] = out["fail"];
+        hrhats[k] = out["hrhat"];
+        psihats[k] = out["psihat"];
+        
+        if (fails[k]) {
+          for (i=0; i<n; i++) {
+            j = index1 + i;
+            boot_indexc[j] = k+1;
+            idc[j] = idb[i];
+            stratumc[j] = stratumb[i];
+            timec[j] = timeb[i];
+            eventc[j] = eventb[i];
+            treatc[j] = treatb[i];
+            rxc[j] = rxb[i];
+            censor_timec[j] = censor_timeb[i];
+            zc_aft(j,_) = zb_aft(i,_);
+          }
+          index1 += n;
+        }
+      }
+      
+      if (is_true(any(fails))) {
+        IntegerVector sub = seq(0,index1-1);
+        boot_indexc = boot_indexc[sub];
+        idc = idc[sub];
+        stratumc = stratumc[sub];
+        timec = timec[sub];
+        eventc = eventc[sub];
+        treatc = treatc[sub];
+        rxc = rxc[sub];
+        censor_timec = censor_timec[sub];
+        zc_aft = subset_matrix_by_row(zc_aft,sub);
+        
+        fail_boots_data = DataFrame::create(
+          Named("boot_index") = boot_indexc,
+          Named("time") = timec,
+          Named("event") = eventc,
+          Named("treated") = treatc,
+          Named("rx") = rxc,
+          Named("censor_time") = censor_timec
+        );
+        
+        for (j=0; j<q+p; j++) {
+          String zj = covariates_aft[j+1];
+          NumericVector u = zc_aft(_,j);
+          fail_boots_data.push_back(u, zj);
+        }
+        
+        if (TYPEOF(data[id]) == INTSXP) {
+          fail_boots_data.push_back(idwi[idc-1], id);
+        } else if (TYPEOF(data[id]) == REALSXP) {
+          fail_boots_data.push_back(idwn[idc-1], id);
+        } else if (TYPEOF(data[id]) == STRSXP) {
+          fail_boots_data.push_back(idwc[idc-1], id);
+        }
+        
+        if (TYPEOF(data[treat]) == LGLSXP || TYPEOF(data[treat]) == INTSXP) {
+          fail_boots_data.push_back(treatwi[1-treatc], treat);
+        } else if (TYPEOF(data[treat]) == REALSXP) {
+          fail_boots_data.push_back(treatwn[1-treatc], treat);
+        } else if (TYPEOF(data[treat]) == STRSXP) {
+          fail_boots_data.push_back(treatwc[1-treatc], treat);
+        }
+        
+        if (has_stratum) {
+          for (i=0; i<p_stratum; i++) {
+            String s = stratum[i];
+            if (TYPEOF(data[s]) == INTSXP) {
+              IntegerVector stratumwi = u_stratum[s];
+              fail_boots_data.push_back(stratumwi[stratumc-1], s);
+            } else if (TYPEOF(data[s]) == REALSXP) {
+              NumericVector stratumwn = u_stratum[s];
+              fail_boots_data.push_back(stratumwn[stratumc-1], s);
+            } else if (TYPEOF(data[s]) == STRSXP) {
+              StringVector stratumwc = u_stratum[s];
+              fail_boots_data.push_back(stratumwc[stratumc-1], s);
+            }
+          }
+        }
+      }
+      
+      // obtain bootstrap confidence interval for HR
+      double loghr = log(hrhat);
+      LogicalVector ok = (1 - fails) & !is_na(hrhats);
+      int n_ok = sum(ok);
+      NumericVector loghrs = log(hrhats[ok]);
+      double sdloghr = sd(loghrs);
+      double tcrit = R::qt(1-alpha/2, n_ok-1, 1, 0);
+      hrlower = exp(loghr - tcrit*sdloghr);
+      hrupper = exp(loghr + tcrit*sdloghr);
+      hr_CI_type = "bootstrap";
+      pvalue = 2*(1 - R::pt(fabs(loghr/sdloghr), n_ok-1, 1, 0));
+      
+      // obtain bootstrap confidence interval for psi
+      NumericVector psihats1 = psihats[ok];
+      double sdpsi = sd(psihats1);
+      psilower = psihat - tcrit*sdpsi;
+      psiupper = psihat + tcrit*sdpsi;
+      psi_CI_type = "bootstrap";
+    }
   }
-
+  
   List settings = List::create(
     Named("psi_test") = psi_test,
     Named("aft_dist") = aft_dist,
@@ -773,7 +936,7 @@ List rpsftmcpp(const DataFrame data,
     Named("boot") = boot,
     Named("n_boot") = n_boot,
     Named("seed") = seed);
-
+  
   List result = List::create(
     Named("psi") = psihat,
     Named("psi_CI") = NumericVector::create(psilower, psiupper),
@@ -789,12 +952,16 @@ List rpsftmcpp(const DataFrame data,
     Named("data_outcome") = data_outcome,
     Named("fit_outcome") = fit_outcome,
     Named("fail") = fail,
+    Named("psimissing") = psimissing,
     Named("settings") = settings);
-
+  
   if (boot) {
     result.push_back(fails, "fail_boots");
     result.push_back(hrhats, "hr_boots");
     result.push_back(psihats, "psi_boots");
+    if (is_true(any(fails))) {
+      result.push_back(fail_boots_data, "fail_boots_data");
+    }
   }
   
   return result;
