@@ -3,60 +3,69 @@ library(splines)
 library(survival)
 
 testthat::test_that("ipcw: pooled logistic regression switching model", {
-  sim1 <- tsegestsim(
-    n = 500, allocation1 = 2, allocation2 = 1, pbprog = 0.5, 
-    trtlghr = -0.5, bprogsl = 0.3, shape1 = 1.8, 
-    scale1 = 360, shape2 = 1.7, scale2 = 688, 
-    pmix = 0.5, admin = 5000, pcatnotrtbprog = 0.5, 
-    pcattrtbprog = 0.25, pcatnotrt = 0.2, pcattrt = 0.1, 
-    catmult = 0.5, tdxo = 1, ppoor = 0.1, pgood = 0.04, 
-    ppoormet = 0.4, pgoodmet = 0.2, xomult = 1.4188308, 
-    milestone = 546, outputRawDataset = 1, seed = 2000)
+  sim1 <- tssim(
+    tdxo = 1, coxo = 1, allocation1 = 1, allocation2 = 1,
+    p_X_1 = 0.3, p_X_0 = 0.3, 
+    rate_T = 0.002, beta1 = -0.5, beta2 = 0.3, 
+    gamma0 = 0.3, gamma1 = -0.9, gamma2 = 0.7, gamma3 = 1.1, gamma4 = -0.8,
+    zeta0 = -3.5, zeta1 = 0.5, zeta2 = 0.2, zeta3 = -0.4, 
+    alpha0 = 0.5, alpha1 = 0.5, alpha2 = 0.4, 
+    theta1_1 = -0.4, theta1_0 = -0.4, theta2 = 0.2,
+    rate_C = 0.0000855,  accrualIntensity = 20/30, 
+    followupTime = 600, fixedFollowup = 0, days = 30,
+    n = 500, NSim = 100, seed = 314159)
+  
+  data1 <- sim1[[1]] %>% 
+    mutate(ostime = timeOS, event = Y)
   
   fit1 <- ipcw(
-    sim1$paneldata, id = "id", tstart = "tstart", 
-    tstop = "tstop", event = "event", treat = "trtrand", 
-    swtrt = "xo", swtrt_time = "xotime", base_cov = "bprog", 
-    numerator = "bprog", denominator = "bprog*catlag", 
+    data1, id = "id", tstart = "tstart", 
+    tstop = "tstop", event = "Y", treat = "trtrand", 
+    swtrt = "xo", swtrt_time = "xotime", 
+    base_cov = "bprog", numerator = "bprog", 
+    denominator = c("bprog", "L"),
     logistic_switching_model = TRUE, ns_df = 3,
     swtrt_control_only = TRUE, boot = FALSE)
   
   # exclude observations after treatment switch
-  data1 <- sim1$paneldata %>%
-    filter(xo == 0 | tstart < xotime) %>%
+  data2 <- data1 %>%
+    filter(ifelse(xo == 1, tstart < xotime, tstop < ostime)) %>%
     group_by(id) %>%
     mutate(condition = row_number() == n() & xo == 1 & tstop >= xotime,
-           cross = ifelse(condition, 1, 0),
-           event = ifelse(condition, 0, event),
-           tstop = ifelse(condition, xotime, tstop))
+           cross = ifelse(condition, 1, 0)) %>%
+    ungroup()
   
   # fit pooled logistic regression switching models
-  data2 <- data1 %>% filter(trtrand == 0)
+  data3 <- data2 %>% filter(trtrand == 0)
   
-  ns1 <- ns(data2$tstop[data2$cross == 1], df = 3)
-  ns2 <- ns(data2$tstop, knots = attr(ns1, "knots"), 
-            Boundary.knots = attr(ns1, "Boundary.knots"))
-  switch1 <- glm(cross ~ bprog*catlag + ns2, family = binomial, data = data2)
-  phat1 <- as.numeric(predict(switch1, newdata = data2, type = "response"))
+  s1 <- ns(data3$tstop[data3$cross == 1], df = 3)
+  s2 <- ns(data3$tstop, knots = attr(s1, "knots"), 
+           Boundary.knots = attr(s1, "Boundary.knots"))
+  switch1 <- glm(cross ~ bprog + L + s2, 
+                 family = binomial, data = data3)
+  phat1 <- as.numeric(predict(switch1, newdata = data3, type = "response"))
   
-  switch2 <- glm(cross ~ bprog + ns2, family = binomial, data = data2)
-  phat2 <- as.numeric(predict(switch2, newdata = data2, type = "response"))
+  switch2 <- glm(cross ~ bprog + s2, family = binomial, data = data3)
+  phat2 <- as.numeric(predict(switch2, newdata = data3, type = "response"))
   
   # stablized weights
-  data3 <- data1 %>% 
+  data4 <- data1 %>% 
     filter(trtrand == 0) %>% 
-    select(id, trtrand, bprog, tstart, tstop, event) %>%
-    left_join(data2 %>% 
-                ungroup() %>%
-                mutate(pstay_den = 1-phat1, pstay_num = 1-phat2) %>%
-                select(id, tstop, pstay_den, pstay_num), 
-              by = c("id", "tstop")) %>%
-    mutate(pstay_den = ifelse(is.na(pstay_den), 1, pstay_den),
-           pstay_num = ifelse(is.na(pstay_num), 1, pstay_num)) %>%
+    select(id, trtrand, bprog, tstart, tstop, event, xo, xotime) %>%
+    left_join(data3 %>% 
+                mutate(tstart = tstop,
+                       o_den = ifelse(cross, phat1, 1-phat1), 
+                       o_num = ifelse(cross, phat2, 1-phat2)) %>%
+                select(id, tstart, o_den, o_num), 
+              by = c("id", "tstart")) %>%
+    mutate(o_den = ifelse(is.na(o_den), 1, o_den),
+           o_num = ifelse(is.na(o_num), 1, o_num)) %>%
     group_by(id) %>%
-    mutate(surv_den = cumprod(pstay_den), surv_num = cumprod(pstay_num),
-           stabilized_weight = surv_num/surv_den) %>%
+    mutate(p_den = cumprod(o_den), p_num = cumprod(o_num),
+           stabilized_weight = p_num/p_den) %>%
+    filter(ifelse(xo == 1, tstart < xotime, TRUE)) %>%
     select(id, tstart, tstop, event, stabilized_weight, bprog, trtrand) %>%
+    ungroup() %>%
     bind_rows(data1 %>% 
                 filter(trtrand == 1) %>%
                 mutate(stabilized_weight = 1) %>%
@@ -64,12 +73,12 @@ testthat::test_that("ipcw: pooled logistic regression switching model", {
                        stabilized_weight, bprog, trtrand))
   
   fit <- coxph(Surv(tstart, tstop, event) ~ trtrand + bprog, 
-               data = data3, weight = stabilized_weight,
+               data = data4, weight = stabilized_weight,
                 id = id, ties = "efron", robust = TRUE)
   
   hr1 <- as.numeric(exp(cbind(fit$coefficients, confint(fit)))["trtrand",])
   
-  testthat::expect_equal(data3$stabilized_weight, 
+  testthat::expect_equal(data4$stabilized_weight, 
                          fit1$data_outcome$stabilized_weight)
   
   testthat::expect_equal(hr1, c(fit1$hr, fit1$hr_CI))
@@ -90,7 +99,7 @@ testthat::test_that("ipcw: time-dependent covariates Cox switching model", {
   # exclude observations after treatment switch
   data1 <- shilong %>%
     mutate(treated = ifelse(bras.f == "CT", 0, 1)) %>%
-    filter(!co | tstart < dco) %>%
+    filter(ifelse(co, tstart < dco, TRUE)) %>%
     arrange(treated, id) %>%
     group_by(treated, id) %>%
     mutate(condition = row_number() == n() & co & tstop >= dco,
